/****************************************************************************\
 *           Copyright (C) 2011, 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
  ****************************************************************************
This example shows data transfer over AIF2 between two DSP on Dual C6670 EVM.
The examples can also be run on C6670 or TCI6614 EVM, but only internal loopback 
test can be run on those boards because there is only one DSP on them.
This file includes code for sending packets and processing RX packets,
TX/RX packets statistics information is collected and summarized
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            August 10, 2011                                                 *
Updated on June 18, 2012 for TCI6614 by Brighton
***************************************************************************/
#include <c6x.h>
#include <stdio.h>
#include <string.h>
#include "KeyStone_common.h"
#include "aif_test_config.h"
#include "aif_PktDMA_init.h"
#include "aif_Data.h"
#include "aif_debug.h"
#include "aif_setup.h"
#include "aif_intc.h"

/*number of bytes be verified (to simplify the test and avoid overloading the DSP, 
only part of the data be verfied)*/
#define NUM_BYTE_VERIFY 	512

#define DEFAULT_IN_BUF_VALUE 	0x35251505

//antenna buffer in memory simulates the buffer of RAC for test
#pragma DATA_ALIGN(RAC_In_Buf, 128) 	/*align to L2 cache line*/
#pragma DATA_SECTION(RAC_In_Buf,".far:RACData")
#pragma DATA_SECTION(RAC_Out_Buf,".far:RACData")
Uint32 RAC_In_Buf[4][64][8]; 	/*32 chips x 64 antennas*/
Uint32 RAC_Out_Buf[4][64][8]; 	/*32 chips x 64 antennas*/

//antenna buffer in memory simulates the buffer of TAC for test
#pragma DATA_ALIGN(TAC_In_Buf, 128) 	/*align to L2 cache line*/
#pragma DATA_SECTION(TAC_In_Buf,".far:TACData")
#pragma DATA_SECTION(TAC_Out_Buf,".far:TACData")
Uint32 TAC_In_Buf[64][4]; 	/*4 chips x 64 antennas*/
Uint32 TAC_Out_Buf[64][4]; 	/*4 chips x 64 antennas*/

/*antenna buffer in memory is arranged antenna by antenna 
for DSP core or RSA processing
this buffer size are limited by some facts:
1. the size of a frame must be multiple of the size of the buffer because 
   DIO will reload parameters every frame
2. DIO blocks number is configured with 13-bit field
3. DIO block address stride is configured with 12-bit field*/
#pragma DATA_ALIGN(Core_In_Buf, 128) 	/*align to L2 cache line*/
#pragma DATA_SECTION(Core_In_Buf,".far:AntData")
#pragma DATA_SECTION(Core_Out_Buf,".far:AntData")
Uint32 Core_In_Buf[NUM_AxC_CHANNEL][NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP];
Uint32 Core_Out_Buf[NUM_AxC_CHANNEL][NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP];

//Aif state for all channels
AifChStat      aifChStat[NUM_AIF_CHANNEL+1];

int (*ant_data_process_fun[3])(Uint32 uiAntStreamNum) =
{
	processRacAntData,
	processTacAntData,
	processCoreAntData
};


//Initialize Antenna Data buffer 
void InitAntDataBuf()
{
	Int32 i, j;

	memset(aifChStat, 0, sizeof(aifChStat));

	/*initialize buffer for RAC Test*/
	for(i= 0; i< 4; i++) 	/*block*/
		for(j= 0; j< 64; j++) 	/*antenna*/
		{
			/*fill with antenna index and chip index*/
			RAC_Out_Buf[i][j][0] = (0<<24)|((8*i+1)<<16)|(j<<8)|(8*i+0);
			RAC_Out_Buf[i][j][1] = (0<<24)|((8*i+3)<<16)|(j<<8)|(8*i+2);
			RAC_Out_Buf[i][j][2] = (0<<24)|((8*i+5)<<16)|(j<<8)|(8*i+4);
			RAC_Out_Buf[i][j][3] = (0<<24)|((8*i+7)<<16)|(j<<8)|(8*i+6);
			/*the second sample*/
			RAC_Out_Buf[i][j][4] = (1<<24)|((8*i+1)<<16)|(j<<8)|(8*i+0);
			RAC_Out_Buf[i][j][5] = (1<<24)|((8*i+3)<<16)|(j<<8)|(8*i+2);
			RAC_Out_Buf[i][j][6] = (1<<24)|((8*i+5)<<16)|(j<<8)|(8*i+4);
			RAC_Out_Buf[i][j][7] = (1<<24)|((8*i+7)<<16)|(j<<8)|(8*i+6);

			RAC_In_Buf[i][j][0] = DEFAULT_IN_BUF_VALUE;
			RAC_In_Buf[i][j][1] = DEFAULT_IN_BUF_VALUE;
			RAC_In_Buf[i][j][2] = DEFAULT_IN_BUF_VALUE;
			RAC_In_Buf[i][j][3] = DEFAULT_IN_BUF_VALUE;
			RAC_In_Buf[i][j][4] = DEFAULT_IN_BUF_VALUE;
			RAC_In_Buf[i][j][5] = DEFAULT_IN_BUF_VALUE;
			RAC_In_Buf[i][j][6] = DEFAULT_IN_BUF_VALUE;
			RAC_In_Buf[i][j][7] = DEFAULT_IN_BUF_VALUE;
		}

	WritebackCache(RAC_Out_Buf, sizeof(RAC_Out_Buf));
	WritebackCache(RAC_In_Buf, sizeof(RAC_In_Buf));

	/*initialize buffer for TAC Test*/
	for(j= 0; j< 64; j++) 	/*antenna*/
	{
		/*fill with antenna index and chip index*/
		TAC_Out_Buf[j][0] = (j<<16)|(0);
		TAC_Out_Buf[j][1] = (j<<16)|(1);
		TAC_Out_Buf[j][2] = (j<<16)|(2);
		TAC_Out_Buf[j][3] = (j<<16)|(3);

		TAC_In_Buf[j][0] = DEFAULT_IN_BUF_VALUE;
		TAC_In_Buf[j][1] = DEFAULT_IN_BUF_VALUE;
		TAC_In_Buf[j][2] = DEFAULT_IN_BUF_VALUE;
		TAC_In_Buf[j][3] = DEFAULT_IN_BUF_VALUE;
	}

	WritebackCache(TAC_Out_Buf, sizeof(TAC_Out_Buf));
	WritebackCache(TAC_In_Buf, sizeof(TAC_In_Buf));

	/*initialize buffer for DSP core Test*/
	for(j= 0; j< NUM_AxC_CHANNEL; j++) 	/*antenna*/
	{
#ifdef CORE_DATA_TYPE_RSA 	/*7 or 8 bit RSA type data*/
		for(i=0; i<NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP/8; i++)
		{
			/*fill with antenna index and chip index*/
			Core_Out_Buf[j][i*8+0] = (0<<24)|(((8*i+1)&0x3f)<<16)|(j<<8)|((8*i+0)&0x3f);
			Core_Out_Buf[j][i*8+1] = (0<<24)|(((8*i+3)&0x3f)<<16)|(j<<8)|((8*i+2)&0x3f);
			Core_Out_Buf[j][i*8+2] = (0<<24)|(((8*i+5)&0x3f)<<16)|(j<<8)|((8*i+4)&0x3f);
			Core_Out_Buf[j][i*8+3] = (0<<24)|(((8*i+7)&0x3f)<<16)|(j<<8)|((8*i+6)&0x3f);
                        /*the second sample*/
			Core_Out_Buf[j][i*8+4] = (1<<24)|(((8*i+1)&0x3f)<<16)|(j<<8)|((8*i+0)&0x3f);
			Core_Out_Buf[j][i*8+5] = (1<<24)|(((8*i+3)&0x3f)<<16)|(j<<8)|((8*i+2)&0x3f);
			Core_Out_Buf[j][i*8+6] = (1<<24)|(((8*i+5)&0x3f)<<16)|(j<<8)|((8*i+4)&0x3f);
			Core_Out_Buf[j][i*8+7] = (1<<24)|(((8*i+7)&0x3f)<<16)|(j<<8)|((8*i+6)&0x3f);
		}
#else 	/*15 or 16 bit normal data type*/
		for(i=0; i<NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP; i++)
		{
			/*fill with antenna index and chip index*/
			Core_Out_Buf[j][i] = (j<<16)|(i&0x3FFF);
		}
#endif
		for(i=0; i<NUM_SLOTS_IN_BUF*WCDMA_SLOT_NUM_CHIP; i++)
		{
			Core_In_Buf[j][i] = DEFAULT_IN_BUF_VALUE;
		}
	}
	
	WritebackCache(Core_Out_Buf, sizeof(Core_Out_Buf));
	WritebackCache(Core_In_Buf, sizeof(Core_In_Buf));

}

//process antenna data for DSP core, return error
int processCoreAntData(Uint32 uiAntStreamNum)
{
	Uint32 uiLastSlotIndex;
	unsigned long long * rxData, * txData;
	unsigned long long ullError= 0;
	int i;

	/*check rx data against tx data*/
	uiLastSlotIndex= (uiSlotCount-1)%NUM_SLOTS_IN_BUF;
	txData = (unsigned long long *)
		&Core_Out_Buf[uiAntStreamNum][uiLastSlotIndex*WCDMA_SLOT_NUM_CHIP];

	rxData = (unsigned long long *)
		&Core_In_Buf[uiAntStreamNum][uiLastSlotIndex*WCDMA_SLOT_NUM_CHIP];

	/*to simplify the test and avoid overloading the DSP, 
	only part of the data be verfied*/
	InvalidCache((void *)rxData, NUM_BYTE_VERIFY);
	for(i=0; i< NUM_BYTE_VERIFY/8; i++)
	{
		//error = sum of (rx-tx)
		ullError = _dadd(ullError,_dsub(rxData[i], txData[i]));
	}

#if 0 	//for debug
	if((0 != ullError)&&(uiSlotCount>1024))
		printf("TX 0x%016llx, RX 0x%016llx; TX 0x%016llx, RX 0x%016llx\n",
			txData[0], rxData[0], txData[1], rxData[1]);
#endif

	/*update the data with current slot number for next transfer
	&0x3f3f3f3f to avoid saturation for 7-bit CPRI mode*/
	txData[0]= uiSlotCount&0x3f3f3f3f;
	WritebackCache((void *)txData, 16);

	return (_hill(ullError)+_loll(ullError));
}

/*process antenna data for RAC, return error*/
int processRacAntData(Uint32 uiAntStreamNum)
{
	unsigned long long * rxData, * txData;
	unsigned long long ullError= 0;
	int i,j;

	/*check rx data against tx data*/
	txData = (unsigned long long *)&RAC_Out_Buf[0][uiAntStreamNum][0];
	rxData = (unsigned long long *)&RAC_In_Buf[0][uiAntStreamNum][0];

	for(i=0; i< 4; i++) 	/*block*/
	{
		InvalidCache((void *)rxData, 32);
		for(j=0; j< 4; j++) 	/*words*/
		{
			//error = sum of (rx-tx)
			ullError = _dadd(ullError,_dsub(rxData[j], txData[j]));
		}
#if 0 	//for debug
		if((0 != ullError))
			printf("TX 0x%016llx, RX 0x%016llx; TX 0x%016llx, RX 0x%016llx; TSC=%u\n",
				txData[0], rxData[0], txData[1], rxData[1], TSCL);
#endif

		rxData += 64*4;
		txData += 64*4;
	}

	/*update the data with current slot number for next transfer
	&0x3f3f3f3f to avoid saturation for 7-bit CPRI mode*/
	RAC_Out_Buf[0][uiAntStreamNum][0]= uiSlotCount&0x3f3f3f3f;
	WritebackCache((void *)&RAC_Out_Buf[0][uiAntStreamNum][0], 16);

	return (_hill(ullError)+_loll(ullError));
}

/*process antenna data for TAC, return error*/
int processTacAntData(Uint32 uiAntStreamNum)
{
	unsigned long long * rxData, * txData;
	unsigned long long ullError= 0;

	/*check rx data against tx data*/
	txData = (unsigned long long *)&TAC_Out_Buf[uiAntStreamNum][0];
	rxData = (unsigned long long *)&TAC_In_Buf[uiAntStreamNum][0];

	InvalidCache((void *)rxData, 16);

	//error = sum of (rx-tx)
	ullError = _dadd(ullError,_dsub(rxData[0], txData[0]));
	ullError = _dadd(ullError,_dsub(rxData[1], txData[1]));

#if 0 	//for debug
	if((0 != ullError)/*&&(uiSlotCount>1024)*/)
		printf("TX 0x%016llx, RX 0x%016llx; TX 0x%016llx, RX 0x%016llx\n",
			txData[0], rxData[0], txData[1], rxData[1]);
#endif

	/*update the data with current slot number for next transfer
	&0x3f3f3f3f to avoid saturation for 7-bit CPRI mode*/
	TAC_Out_Buf[uiAntStreamNum][0]= uiSlotCount&0x3f3f3f3f;
	WritebackCache((void *)&TAC_Out_Buf[uiAntStreamNum][0], 16);

	return (_hill(ullError)+_loll(ullError));
}

Uint16 badSlots[1024];
Uint32 badSlotsIndex=0;
/*process antenna data*/
void processAntData()
{
	int i, j;
	int error;
	int uiStreamIndex[3]={0, 0, 0};
	WcdmaAif2TestType 	testType;
	
	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		if(0==aifLinkCfg[i].bPDEnabled)
			continue;

		testType= aifLinkCfg[i].testType;
		//process antenna data channel
		for(j=aifLinkCfg[i].firstAxcNum; 
			j<aifLinkCfg[i].firstAxcNum+ aifLinkCfg[i].numberAxC; j++)
		{
			//call processing function according to test type
			error= ant_data_process_fun[testType](uiStreamIndex[testType]++);
			if(error)
			{
				aifChStat[j].uiNumBadPackets++;
				badSlots[(badSlotsIndex++)&1023]= uiSlotCount;
			}
			else
			{
				aifChStat[j].uiNumGoodPackets++;
			}
			aifChStat[j].ullNumBytes += WCDMA_SLOT_NUM_CHIP*4;
		}
	}
}

Uint32 descRecord[128];
Uint32 descRecordIndex= 0;
/*initialize packets and push them to TX queue*/
void InitSendControlPacket(AifCtlChTestInfo * testInfo)
{
	Uint32 uiDescriptor;
	Uint32 uiTxQ;
	unsigned char * ucpBufPointer;
	MonolithicPacketDescriptor * Descriptor;

	uiDescriptor =KeyStone_queuePop(AIF_TX_FDQ);
	if(0==uiDescriptor)
	{
		printf("Source queue descriptor is NULL for channel %d at frame %d slot %d\n", 
			testInfo->uiChannelNum, uiFrameCount, uiSlotCount);
		AIF_Check_Free_Queues();	//for debug
		AIF_Check_TxRx_Queues(); 	//for debug
		return;
	}

	/*invalid cache before read descriptor RAM*/
	InvalidCache((void *)uiDescriptor, 64);

	Descriptor= (MonolithicPacketDescriptor *) uiDescriptor;
	ucpBufPointer = (unsigned char *)(Descriptor+1);
	Descriptor->data_offset= 
		sizeof(MonolithicPacketDescriptor);

	Descriptor->packet_length= testInfo->uiPacketSize;
	Descriptor->psv_word_count= 0;
	
	/*put the testInfo int the packet,
	it will be verified at the RX side*/
	memcpy(ucpBufPointer, testInfo, sizeof(AifCtlChTestInfo));

	/*write back data from cache to descriptor RAM*/
	WritebackCache((void *)uiDescriptor, 64);

	uiTxQ = QMSS_AIF_QUEUE_BASE+testInfo->uiChannelNum;

	//TSC_delay_us(1);
	descRecord[descRecordIndex++]= uiDescriptor;
	descRecordIndex &= 127;
	
	_mfence(); 	//force all memory operation complete
	_mfence(); 	//force all memory operation complete
	KeyStone_queuePush(uiTxQ, uiDescriptor|FETCH_SIZE_16);

	aifChStat[testInfo->uiChannelNum].uiNumTXPackets++;
}


/*send Control data*/
void sendCtlData()
{
	int i;
	
	AifCtlChTestInfo testInfo;

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		if(0==aifLinkCfg[i].bPDEnabled)
			continue;

		//process control channel
#ifdef AIF2_LINK_PROTOCOL_OBSAI
		if(aifLinkCfg[i].bUseControlMsg)
#else
		if(aifLinkCfg[i].numCtlWord)
#endif
		{
			testInfo.uiLinkNum= i; 	
			testInfo.uiChannelNum= aifLinkCfg[i].controlChNum; 	
			testInfo.uiCtlPktNum= uiCtlPktCount; 	
#ifdef AIF2_LINK_PROTOCOL_OBSAI
			testInfo.uiPacketSize= (WCDMA_CONTROL_PACKET_SIZE>>aifLinkCfg[i].linkRate)/2;
#else
			testInfo.uiPacketSize= 704-i;
				//(WCDMA_CONTROL_PACKET_SIZE>>aifLinkCfg[i].linkRate)*
				//aifLinkCfg[i].numCtlWord/64/4;
#endif
			InitSendControlPacket(&testInfo);
		}
	}
}

/*print test info for debug*/
void printTestInfo(AifCtlChTestInfo * testInfo)
{
	printf("uiLinkNum   =%d, \
uiChannelNum=%d, \
uiPacketSize=%d, \
uiCtlPktNum =%d\n",
	testInfo->uiLinkNum   ,
	testInfo->uiChannelNum,
	testInfo->uiPacketSize,
	testInfo->uiCtlPktNum);
}

/*process received packet*/
void receiveCtlData(Uint32 uiDescriptor)
{
	unsigned char * ucpBufPointer;
	Uint32 uiPacketLength, uiChannelNum, uiErrorFlag;
	AifCtlChTestInfo * testInfo; 
	Uint32 uiReturnQ;
	MonolithicPacketDescriptor * Descriptor;

	/*invalid cache before read descriptor RAM*/
	InvalidCache((void *)uiDescriptor, 64);

	Descriptor= (MonolithicPacketDescriptor *) uiDescriptor;
	ucpBufPointer = (unsigned char *)(uiDescriptor+
		Descriptor->data_offset);

	uiErrorFlag= Descriptor->err_flags;
	uiPacketLength= Descriptor->packet_length;

	/*test information received, need be verified*/
	testInfo = (AifCtlChTestInfo *)ucpBufPointer;
	uiChannelNum= testInfo->uiChannelNum;
	if(uiChannelNum!= aifLinkCfg[testInfo->uiLinkNum].controlChNum)
	{
		//bad control packet are count into additional channel
		uiChannelNum= uiNumAifChannel;
		uiErrorFlag= 1;
	}

	/*verify received information*/
	if(uiErrorFlag||(testInfo->uiPacketSize != uiPacketLength)
			/*||(((Uint32)(uiCtlPktCount - testInfo->uiCtlPktNum))>(QUEUE_ACC_THRESH*2))*/)
	{
		aifChStat[uiChannelNum].uiNumBadPackets++;
		aifChStat[uiChannelNum].ullNumBytes += uiPacketLength;
	}
	else
	{
		aifChStat[uiChannelNum].uiNumGoodPackets++;
		aifChStat[uiChannelNum].ullNumBytes += uiPacketLength;
	}

	/*recycle RX packet buffer*/
	uiReturnQ= (Descriptor->pkt_return_qmgr<<12)|
		Descriptor->pkt_return_qnum;
	KeyStone_queuePush(uiReturnQ, uiDescriptor);
	_mfence(); 	//force all memory operation complete
	_mfence(); 	//force all memory operation complete
}

void PrintChannelResult(Uint32 uiChNum)
{
	aifChStat[uiChNum].uiBandwidth= 
		aifChStat[uiChNum].ullNumBytes/((uiFrameCount-1)*10);
	
	printf("Channel %d transfer %d good packets, %d bad packets, %lld bytes, achieve %d KB/s\n",
		uiChNum, aifChStat[uiChNum].uiNumGoodPackets, aifChStat[uiChNum].uiNumBadPackets, 
		aifChStat[uiChNum].ullNumBytes,	aifChStat[uiChNum].uiBandwidth);
}

void PrintTestResult()
{
	Int32 i, j;
	Uint32 uiTotalBandwidth=0;
	Uint32 uiLinkGoodSlots, uiLinkBadSlots, uiLinkBandwidth;

	print_AIF_status(); 	//for debug
	AIF_Check_Free_Queues();	//for debug
	AIF_Check_TxRx_Queues(); 	//for debug
	//printNonEmptyQueue(); 		//for debug

	for(i=0; i<NUM_AIF2_LINK; i++)	//for all links
	{
		if(0==aifLinkCfg[i].linkEnable)
			continue;

		uiLinkBandwidth= 0;
		uiLinkGoodSlots= 0;
		uiLinkBadSlots= 0;

		for(j=aifLinkCfg[i].firstAxcNum; 
			j<aifLinkCfg[i].firstAxcNum+ aifLinkCfg[i].numberAxC; j++)
		{
			printf("  AxC ");
			PrintChannelResult(j);
			uiLinkBandwidth+= aifChStat[j].uiBandwidth;
			uiLinkGoodSlots+= aifChStat[j].uiNumGoodPackets;
			uiLinkBadSlots+= aifChStat[j].uiNumBadPackets;
		}
		printf(" link %d transfer %d good antenna slots, %d bad antenna slots\n", 
			i, uiLinkGoodSlots, uiLinkBadSlots);

#ifdef AIF2_LINK_PROTOCOL_OBSAI
		if(aifLinkCfg[i].bUseControlMsg)
#else
		if(aifLinkCfg[i].numCtlWord)
#endif
		{
			printf("  control ");
			PrintChannelResult(aifLinkCfg[i].controlChNum);
			uiLinkBandwidth+= aifChStat[aifLinkCfg[i].controlChNum].uiBandwidth;
		}

		printf("Throughput of link %d = %d MB/s\n", i, uiLinkBandwidth/1000);

		uiTotalBandwidth+= uiLinkBandwidth;
	}

	//bad control packet are count into additional channel
	if(aifChStat[uiNumAifChannel].uiNumBadPackets)
	{
		aifChStat[uiNumAifChannel].uiBandwidth= 
			aifChStat[uiNumAifChannel].ullNumBytes/((uiFrameCount-1)*10000);
		
		printf("Totally received %d bad control packets, %lld bytes\n",
			aifChStat[uiNumAifChannel].uiNumBadPackets,
			aifChStat[uiNumAifChannel].ullNumBytes);
	}

	printf("Total throughput of AIF = %d MB/s\n", uiTotalBandwidth/1000);

}


